@@ -36,7 +36,7 @@ import dbus.mainloop.glib
 
 from hamster import dispatcher, storage, SHARED_DATA_DIR
 import hamster.eds
-from hamster.Configuration import GconfStore
+from hamster.configuration import GconfStore
 
 from hamster import stuff
 from hamster.KeyBinder import *
@@ -62,23 +62,27 @@ class Notifier(object):
     def __init__(self, app_name, icon, attach):
         self._icon = icon
         self._attach = attach
-        self._notify = None
-        # Title of reminder baloon
+        # Title of reminder notification
         self.summary = _("Time Tracker")
       
         if not pynotify.is_initted():
             pynotify.init(app_name)
 
     def msg(self, body, edit_cb, switch_cb):
-        self._notify = pynotify.Notification(self.summary, body, self._icon, self._attach)
+        notify = pynotify.Notification(self.summary, body, self._icon, self._attach)
 
         if "actions" in pynotify.get_server_caps():
             #translators: this is edit activity action in the notifier bubble
-            self._notify.add_action("edit", _("Edit"), edit_cb)
+            notify.add_action("edit", _("Edit"), edit_cb)
             #translators: this is switch activity action in the notifier bubble
-            self._notify.add_action("switch", _("Switch"), switch_cb)
+            notify.add_action("switch", _("Switch"), switch_cb)
             
-        self._notify.show()
+        notify.show()
+    
+    def msg_low(self, message):
+        notify = pynotify.Notification(self.summary, message, self._icon, self._attach)
+        notify.set_urgency(pynotify.URGENCY_LOW)
+        notify.show()
 
 
 class PanelButton(gtk.ToggleButton):
@@ -204,8 +208,20 @@ class HamsterApplet(object):
         self._gui = stuff.load_ui_file("applet.ui")
         self.window = self._gui.get_object('hamster-window')
 
+
+
+        # set up autocompletition
+        self.activities = gtk.ListStore(gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING,
+                                        gobject.TYPE_STRING)
+        
+        # build the menu
         self.set_dropdown()
 
+        self.refresh_menu()
+        
+        self.init_autocomplete()
+
         # init today's tree
         self.setup_activity_tree()
 
@@ -229,6 +245,8 @@ class HamsterApplet(object):
         # Load today's data, activities and set label
         self.last_activity = None
 
+
+
         self.load_day()
         self.update_label()
 
@@ -238,8 +256,6 @@ class HamsterApplet(object):
         # refresh hamster every 60 seconds to update duration
         gobject.timeout_add_seconds(60, self.refresh_hamster)
 
-        # build the menu
-        self.refresh_menu()
 
         # remove padding, so we fit on small panels (adapted from clock applet)
         gtk.rc_parse_string ("""style "hamster-applet-button-style" {
@@ -278,7 +294,10 @@ class HamsterApplet(object):
 
         # init idle check
         dispatcher.add_handler('gconf_timeout_enabled_changed', self.on_timeout_enabled_changed)
-        self.on_timeout_enabled_changed(None, self.config.get_timeout_enabled())
+        self.timeout_enabled = self.config.get_timeout_enabled()
+
+        dispatcher.add_handler('gconf_notify_on_idle_changed', self.on_notify_on_idle_changed)
+        self.notify_on_idle = self.config.get_notify_on_idle()
         
         
         # init nagging timeout
@@ -329,6 +348,7 @@ Now, start tracking!
 
         edit_cell = gtk.CellRendererPixbuf()
         edit_cell.set_property("stock_id", "gtk-edit")
+        edit_cell.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
         self.edit_column = gtk.TreeViewColumn("", edit_cell)
         self.treeview.append_column(self.edit_column)
 
@@ -345,6 +365,7 @@ Now, start tracking!
         self.activity_list = self._gui.get_object('activity-list')
         self.activity_list.child.connect('activate', self.on_activity_entered)
         self.activity_list.child.connect('key-press-event', self.on_activity_list_key_pressed)
+        self.activity_list.child.connect('changed', self.on_activity_entry_changed)
 
         self.activity_list.set_model(gtk.ListStore(gobject.TYPE_STRING,
                                                    gobject.TYPE_STRING,
@@ -362,35 +383,115 @@ Now, start tracking!
         self.activity_list.set_property("text-column", 2)
 
 
-        # set up autocompletition
-        self.activities = gtk.ListStore(gobject.TYPE_STRING,
-                                        gobject.TYPE_STRING,
-                                        gobject.TYPE_STRING)
-        completion = gtk.EntryCompletion()
-        completion.set_model(self.activities)
+    def init_autocomplete(self):
+        #fillable, activity_name, category
+        self.autocomplete_store = gtk.ListStore(gobject.TYPE_STRING,
+                                                gobject.TYPE_STRING,
+                                                gobject.TYPE_STRING)
+        self.completion = gtk.EntryCompletion()
+        self.completion.set_model(self.autocomplete_store)
+        self.completion.set_minimum_key_length(1)
+        self.completion.set_inline_completion(True)
+        self.completion.set_popup_set_width(False)
+        self.completion.set_text_column(0)
+        self.activity_list.child.set_completion(self.completion)
+        
+
+    def on_match_selected(self, completion, model, iter):
+        entry = completion.get_entry()
+        activity = stuff.parse_activity_input(entry.get_text())
+        
+        fillable = model.get_value(iter, 0)
+        
+        res = ""
+        
+        if activity.start_time:
+            res += "%s" % activity.start_time.strftime("%H:%M")
+            if activity.end_time:
+                res += "-%s" % activity.start_time.strftime("%H:%M")
+        
+            res += " "
+            #strip time
+            fillable = fillable[fillable.find(" ")+1:]
+        
+        res += fillable
+        
+        entry.set_text(res)
+        entry.set_position(len(res))
+        return True
+    
+    def redo_autocomplete(self):
+        """parses input and generates autocomplete according to what has
+        to be completed now"""
+        #TODO turn this whole thing into a widget
+        self.autocomplete_store.clear()
+        
+        input_text = self.activity_list.child.get_text()
+        
+        if not input_text:
+            return
+        
+        
+        entry = self.activity_list.child
+        parsed_activity = stuff.parse_activity_input(entry.get_text())
+
+        if input_text.find("@") > 0:
+            key = input_text[input_text.find("@")+1:]
+            for category in self.all_categories:
+                if category['name'].startswith(key):
+                    fillable = input_text[:input_text.find("@") + 1] + category['name']
+                    self.autocomplete_store.append([fillable,
+                                                    category['name'],
+                                                    category['name']
+                                                   ])
+            if len(self.autocomplete_store) <= 1:
+                #avoid popup on single result
+                self.autocomplete_store.clear()
+        else:
+            for activity in self.all_activities:
+                fillable = activity['name']
+                if activity['category']:
+                    fillable += "@%s" % activity['category']
+                    
+                if parsed_activity.start_time:
+                    fillable = entry.get_text()[:entry.get_text().find(" ")+1] + fillable
+    
+                self.autocomplete_store.append([fillable,
+                                                activity['name'],
+                                                activity['category']
+                                               ])
+
+                
+        activity = stuff.parse_activity_input(input_text)
+        self.completion.clear()
+        if activity.start_time:
+            bgcolor = gtk.Style().bg[gtk.STATE_NORMAL].to_string()
+            time_cell = gtk.CellRendererPixbuf()
+            time_cell.set_property("icon-name", "appointment-new")
+            time_cell.set_property("cell-background", bgcolor)
+            
+            self.completion.pack_start(time_cell, False)
+
+            time_cell = gtk.CellRendererText()
+            time = activity.start_time.strftime("%H:%M")
+            if activity.end_time:
+                time += "-%s" % activity.end_time.strftime("%H:%M")
+                
+            time_cell.set_property("text", time)
+            time_cell.set_property("scale", 0.8)
+            time_cell.set_property("cell-background", bgcolor)
+            self.completion.pack_start(time_cell, False)
 
         activity_cell = gtk.CellRendererText()
-        completion.pack_start(activity_cell, True)
-        completion.add_attribute(activity_cell, 'text', 0)
-        completion.set_property("text-column", 2)
+        self.completion.pack_start(activity_cell, True)
+        self.completion.add_attribute(activity_cell, 'text', 1)
+        #self.completion.set_property("text-column", 2)
 
-        category_cell = stuff.CategoryCell()  
-        completion.pack_start(category_cell, False)
-        completion.add_attribute(category_cell, 'text', 1)
-
-
-        def match_func(completion, key, iter):
-            model = completion.get_model()
-            text = model.get_value(iter, 2)
-            if text and text.startswith(key):
-                return True
-            return False
-
-        completion.set_match_func(match_func)
-        completion.set_minimum_key_length(1)
-        completion.set_inline_completion(True)
-
-        self.activity_list.child.set_completion(completion)
+        if input_text.find("@") == -1:
+            #no need for category cell when we populate categories
+            category_cell = stuff.CategoryCell()  
+            self.completion.pack_start(category_cell, False)
+            self.completion.add_attribute(category_cell, 'text', 2)
         
 
     def on_today_release_event(self, tree, event):
@@ -398,12 +499,7 @@ Now, start tracking!
         path = tree.get_path_at_pos(pointer[0], pointer[1]) #column, innerx, innery
         
         if path and path[1] == self.edit_column:
-            selection = tree.get_selection()
-            (model, iter) = selection.get_selected()
-            fact_id = model[iter][0]
-                
-            custom_fact = CustomFactController(self, None, fact_id)
-            custom_fact.show()
+            self._open_edit_activity()
             return True
         
         return False
@@ -413,7 +509,7 @@ Now, start tracking!
         """refresh hamster every x secs - load today, check last activity etc."""        
         # stop tracking task if computer is idle for X minutes
         if self.timeout_enabled and self.last_activity and \
-           self.last_activity['end_time'] == None:
+           self.last_activity['end_time'] is None:
             if self.dbusIdleListener.is_idle:
                 # Only subtract idle time from the running task when
                 # idleness is due to time out, not a screen lock.
@@ -426,21 +522,16 @@ Now, start tracking!
                 storage.touch_fact(self.last_activity, end_time = idle_from)
             
 
-        if self.last_activity and self.last_activity['end_time'] == None:
-            # if we have running task and nagging is enabled
-            # check if maybe it is time to nag
-            if self.notify_interval:
-                self.check_user()
-
             # if we have date change - let's finish previous task and start a new one
             if self.button.get_active(): # otherwise if we the day view is visible - update day's durations
                 self.load_day()
                     
         self.update_label()
+        self.check_user()
         return True
 
     def update_label(self):
-        if self.last_activity and self.last_activity['end_time'] == None:
+        if self.last_activity and self.last_activity['end_time'] is None:
             delta = datetime.datetime.now() - self.last_activity['start_time']
             duration = delta.seconds /  60
             label = "%s %s" % (self.last_activity['name'],
@@ -459,13 +550,22 @@ Now, start tracking!
         self.dbusController.update_activity(label)
 
     def check_user(self):
-        delta = datetime.datetime.now() - self.last_activity['start_time']
-        duration = delta.seconds /  60
-                 
-        if duration and duration % self.notify_interval == 0:
-            # activity reminder
-            msg = _(u"Working on <b>%s</b>") % self.last_activity['name']
-            self.notify.msg(msg, self.edit_cb, self.switch_cb)
+        if not self.notify_interval: #no interval means "never"
+            return
+        
+        now = datetime.datetime.now()
+        if self.last_activity:
+            delta = now - self.last_activity['start_time']
+            duration = delta.seconds /  60
+
+            if duration and duration % self.notify_interval == 0:
+                # activity reminder
+                msg = _(u"Working on <b>%s</b>") % self.last_activity['name']
+                self.notify.msg(msg, self.edit_cb, self.switch_cb)
+        elif self.notify_on_idle:
+            #if we have no last activity, let's just calculate duration from 00:00
+            if (now.minute + now.hour *60) % self.notify_interval == 0:
+                self.notify.msg_low(_(u"No activity"))
 
     def edit_cb(self, n, action):
         custom_fact = CustomFactController(self, None, self.last_activity['id'])
@@ -542,7 +642,11 @@ Now, start tracking!
     def refresh_menu(self):
         #first populate the autocomplete - contains all entries in lowercase
         self.activities.clear()
-        all_activities = storage.get_autocomplete_activities()
+        self.all_activities = all_activities = storage.get_autocomplete_activities()
+        self.all_categories = storage.get_category_list()
+        
+        
+        
         for activity in all_activities:
             activity_category = activity['name']
             if activity['category']:
@@ -594,7 +698,7 @@ Now, start tracking!
 
     def __update_fact(self):
         """dbus controller current fact updating"""
-        if self.last_activity and self.last_activity['end_time'] == None:
+        if self.last_activity and self.last_activity['end_time'] is None:
             self.dbusController.update_fact(self.last_activity["name"])
         else:
             self.dbusController.update_fact(_(u'No activity'))
@@ -603,6 +707,20 @@ Now, start tracking!
     def __show_toggle(self, event, is_active):
         """main window display and positioning"""
         self.button.set_active(is_active)
+
+        if self.last_activity and self.last_activity["end_time"] is None:
+            label = self.last_activity['name']
+            if self.last_activity['category'] != _("Unsorted"):
+                label += "@%s" %  self.last_activity['category']
+            self.activity_list.child.set_text(label)
+
+            self.activity_list.child.select_region(0, -1)
+            self._gui.get_object("more_info_label").hide()
+        else:
+            self.activity_list.child.set_text('')
+            self._gui.get_object("more_info_label").show()
+
+
         if not is_active:
             self.window.hide()
             return
@@ -628,17 +746,6 @@ Now, start tracking!
         self.window.move(x, y)
 
 
-        if self.last_activity and self.last_activity["end_time"] == None:
-            label = self.last_activity['name']
-            if self.last_activity['category'] != _("Unsorted"):
-                label += "@%s" %  self.last_activity['category']
-            self.activity_list.child.set_text(label)
-
-            self.activity_list.child.select_region(0, -1)
-            self._gui.get_object("more_info_label").hide()
-        else:
-            self.activity_list.child.set_text('')
-            self._gui.get_object("more_info_label").show()
 
         # doing unstick / stick here, because sometimes while switching
         # between workplaces window still manages to dissappear
@@ -667,6 +774,11 @@ Now, start tracking!
         #treating tab as keydown to be able to cycle through available values
         if event.keyval == gtk.keysyms.Tab:
             event.keyval = gtk.keysyms.Down
+        
+        return False
+    
+    def on_activity_entry_changed(self, entry):
+        self.redo_autocomplete()
         return False
         
     def on_activity_switched(self, component):
@@ -687,19 +799,41 @@ Now, start tracking!
         dispatcher.dispatch('panel_visible', False)
 
     """listview events"""
-    def on_todays_keys(self, tree, event_key):
-        if (event_key.keyval == gtk.keysyms.Delete):
+    def on_todays_keys(self, tree, event):
+        if (event.keyval == gtk.keysyms.Delete):
             self.delete_selected()
             return True
+        elif (event.keyval == gtk.keysyms.e  \
+              and event.state & gtk.gdk.CONTROL_MASK):
+            self._open_edit_activity()
+            return True
+            
         return False
     
-    def on_today_row_activated(self, tree, path, column):
-        selection = tree.get_selection()
+    def _open_edit_activity(self):
+        """opens activity editor for selected row"""
+        selection = self.treeview.get_selection()
         (model, iter) = selection.get_selected()
-        activity_name = model[iter][1].decode('utf8', 'replace')
-        if activity_name:
-            storage.add_fact(activity_name)
-            dispatcher.dispatch('panel_visible', False)        
+        fact_id = model[iter][0]
+            
+        custom_fact = CustomFactController(self, None, fact_id)
+        custom_fact.show()        
+    
+    def on_today_row_activated(self, tree, path, column):
+        if column == self.edit_column:
+            self._open_edit_activity()
+        else:
+            selection = tree.get_selection()
+            (model, iter) = selection.get_selected()
+            activity_name = model[iter][1].decode('utf8', 'replace')
+            if activity_name:
+                description = model[iter][5]
+                if description:
+                    description = description.decode('utf8', 'replace')
+                    activity_name = "%s, %s" % (activity_name, description)
+                    
+                storage.add_fact(activity_name)
+                dispatcher.dispatch('panel_visible', False)
         
         
     def on_windows_keys(self, tree, event_key):
@@ -764,6 +898,10 @@ Now, start tracking!
         # if enabled, set to value, otherwise set to zero, which means disable
         self.timeout_enabled = enabled
 
+    def on_notify_on_idle_changed(self, event, enabled):
+        # if enabled, set to value, otherwise set to zero, which means disable
+        self.notify_on_idle = enabled
+
     def on_notify_interval_changed(self, event, new_interval):
         if PYNOTIFY and 0 < new_interval < 121:
             self.notify_interval = new_interval
